#include <mem.h>
#include <conio.h>
#include <stdio.h>

#include "include\prototyp.h"
#include "include\global.h"
#include "include\couleur.h"
#include "include\keyboard.h"

/*************************************************************************/
/* Nom : retourne                                                        */
/*                                                                       */
/* Role : Fonction permettant de determiner si le placement est autorise */
/*        et/ou permet de retourner les pions                            */
/*                                                                       */
/*                                                                       */
/* Entree : damier : damier du jeu sur lequel on va travailler           */
/*          cx, cy : indices du pion place                               */
/*          flip   : retournement des pions autorises                    */
/*          joueurCourant : joueur pour lequel on analyse le placement   */
/*                                                                       */
/* Sortie : choix de l'emplacement autorise                              */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 30/01/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 21/02/1997 | Rajout de deux parametres joueurAdverse et         */
/*     |            | joueurCourant pour faciliter les traitements       */
/*     |            | dans le cas ou l'on veut verifier si les joueurs   */
/*     |            | doivent passer leur tour                           */
/*     |            |                                                    */
/* PDB | 02/03/1997 | modification des parametres                        */
/*     |            |                                                    */
/*************************************************************************/


char retourne (char damier[10][10], char choixX, char choixY, char flip, int joueurCourant)
{
	int  xDir, yDir; 	    /* direction de la recherche */
	int  caseX, caseY;          /* case courante de la recherche */
	int  i;			    /* compteur de boucle */
	int  compteurEtape;         /* etape dans la recherche */
	int  joueurAdverse;	    /* joueur adverse pour la recherche */
	char choixAutorise = FALSE; /* emplacempent autorise */

	/* on determine le joueur adverse */
	joueurAdverse = -joueurCourant + 3;

	/* deja on regarde si la case choisie est libre */
	if (damier[choixX][choixY] == VIDE)
	{
		/* on regarde dans toute les directions */
		for (xDir=-1; xDir < 2; xDir++)
		{
			for (yDir=-1; yDir < 2; yDir++)
			{
				compteurEtape = 0;

				/* tant qu'il y a des cases dans la direction x et y */
				do
				{
					compteurEtape++;
					caseX = choixX + compteurEtape*xDir;
					caseY = choixY + compteurEtape*yDir;

				} while ((caseX > 0) && (caseX < 9) &&
					 (caseY > 0) && (caseY < 9) &&
					 (damier[caseX][caseY] == joueurAdverse));

				/* le choix est autorise si dans la recherche on trouve une couleur entouree par l'autre */
				if ((caseX > 0) && (caseX < 9) &&
				    (caseY > 0) && (caseY < 9) &&
				    (compteurEtape > 1)        &&
				    (damier[caseX][caseY] == joueurCourant))
				{
					choixAutorise = TRUE;

					/* on retourne les pions si cela a ete demande */
					if (flip)
						for (i=1; i<compteurEtape; i++)
							damier[choixX+xDir*i][choixY+yDir*i] = joueurCourant;

				}

			}
		}
	}
	return choixAutorise;
}

/*************************************************************************/
/* Nom : coup_possibles                                                  */
/*                                                                       */
/* Role : Fonction permettant de rechercher les coups possibles          */
/*                                                                       */
/*                                                                       */
/* Entree : damier        : damier a analyser                            */
/*          possibles     : coups possibles du damier                    */
/*          joueurCourant : joueur courant                               */
/*                                                                       */
/* Sortie : nombre de coups possibles                                    */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 02/03/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 16/04/1997 | Optimisation de la recherche du nombre de coups    */
/*     |            | possibles                                          */
/*     |            |                                                    */
/*************************************************************************/

int coup_possibles (char               board[10][10],
		    typeStructureCoups possibles[NOMBRE_COUPS],
		    int                joueurCourant)
{
	char caseX, caseY ; /* emplacement du pion sur le damier */
	int  coups = 0    ; /* nombre de coups */

	int  minX = 8,
	     minY = 8,
	     maxX = 1,
	     maxY = 1;     /* variables utilisees pour optimiser la recherche */

	int joueurAdverse ; /* joueur adversaire */


	/* on determine le joueur adverse */
	joueurAdverse = -joueurCourant + 3;


	/* On determine les bornes pour la recherche des coups dans un damier minimise */
	/* on arrive a avoir un gain 5x plus rapide au debut du jeu                    */
	/* et en fin de jeu le gain est nul                                            */

	for (caseX = 1; caseX < 9; caseX++)
	{
		for (caseY = 1; caseY < 9; caseY++)
		{
			if (board[caseX][caseY] == joueurAdverse)
			{
				if (minX > caseX)
					minX = caseX;

				if (maxX < caseX)
					maxX = caseX;

				if (minY > caseY)
					minY = caseY;

				if (maxY < caseY)
					maxY = caseY;
			}
		}
	}


	/* on affine la recherche */
	if (minX != 1)
		minX--;

	if (maxX != 8)
		maxX++;

	if (minY != 1)
		minY--;

	if (maxY != 8)
		maxY++;

	/* on determine si le joueur courant peut poser un pion */
	for (caseX = minX; caseX <= maxX; caseX++)
		for (caseY = minY; caseY <= maxY; caseY++)
			if (retourne(board, caseX, caseY, FALSE, joueurCourant))
			{
				possibles[coups].x = caseX;
				possibles[coups].y = caseY;
				coups++;
			}

	/* Retourne le nombre de coups possibles */
	return coups;
}

/*************************************************************************/
/* Nom : evaluation_damier                                               */
/*                                                                       */
/* Role : Fonction permettant d'valuer un damier afin de determiner     */
/*        la situation du jeu en lui donnant une note tel que :          */
/*                1 < note < +maxNote le jeu est favorable aux noirs     */
/*         -maxNote < note < -1       le jeu est defavorable aux noirs,  */
/*                                    donc favorable aux blanc           */
/*                                                                       */
/*                                                                       */
/* Entree : damier        : damier a evaluer                             */
/*                                                                       */
/* Sortie : note du damier                                               */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 02/03/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 18/04/1997 | Amelioration de l'evaluation du damier             */
/*     |            |                                                    */
/*     |            |                                                    */
/*************************************************************************/

int evaluation_damier ( char damier[10][10] )
{
	/* on attribut des lettres a chaque case du 	*/
	/* damier                                       */
	/*						*/
	/*   a b c d e f g h 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 1| |C|A|B|B|A|C| | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 2|C|X|F|G|G|F|X|C| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 3|A|F|D|E|E|D|F|A| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 4|B|G|E| | |E|G|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 5|B|G|E| | |E|G|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 6|A|F|D|E|E|D|F|A| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 7|C|X|F|G|G|F|X|C| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 8| |C|A|B|B|A|C| | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/*						*/
	/* on attribut une note en fonction du type de  */
	/* de la case                                   */
	/* la note sera attribue de facon decroissante  */
	/* dans l'ordre suivant pour le joueur :        */
	/* - les coins                                  */
	/* - puis les cases A-D-B-E-G-F-C-X		*/
	/* on procede de facon inverse pour determiner  */
	/* la note de l'adversaire                      */


	/* score d'une case jouee par un joueur noir */

	int scoreJoueur [10][10] = {
		{ 0,    0,    0,   0,  0,  0,   0,    0,    0, 0 }, /* ligne 0 */
		{ 0,  900, -220,  90, 70, 70,  90, -220,  900, 0 }, /* ligne 1 */
		{ 0, -220, -200, -20,  0,  0, -20, -200, -220, 0 }, /* ligne 2 */
		{ 0,   90, - 30,  30, 40, 20,  40, - 30,   90, 0 }, /* ligne 3 */
		{ 0,   78,    0,  40,  0,  0,  20,    0,   78, 0 }, /* ligne 4 */
		{ 0,   78,    0,  20,  0,  0,  40,    0,   78, 0 }, /* ligne 5 */
		{ 0,   90, - 30,  40, 20, 40,  30, - 30,   90, 0 }, /* ligne 6 */
		{ 0, -220, -200, -20,  0,  0, -20, -200, -220, 0 }, /* ligne 7 */
		{ 0,  900, -220,  90, 70, 70,  90, -220,  900, 0 }, /* ligne 8 */
		{ 0,    0,    0,   0,  0,  0,   0,    0,    0, 0 }, /* ligne 9 */
	};

//	int scoreJoueur [10][10] = {
//		{ 0,   0,   0,   0,   0,   0,   0,   0,   0, 0 }, /* ligne 0 */
//		{ 0,  50,   4,  16,  12,  12,  16,   4,  50, 0 }, /* ligne 1 */
//		{ 0,   4, -30,  -4,  -5,  -5,  -4, -30,   4, 0 }, /* ligne 2 */
//		{ 0,  16,  -4,   1,   0,   0,   1,  -4,  16, 0 }, /* ligne 3 */
//		{ 0,  12,  -5,   0,   0,   0,   0,  -5,  12, 0 }, /* ligne 4 */
//		{ 0,  12,  -5,   0,   0,   0,   0,  -5,  12, 0 }, /* ligne 5 */
//		{ 0,  16,  -4,   1,   0,   0,   1,  -4,  16, 0 }, /* ligne 6 */
//		{ 0,   4, -30,  -4,  -5,  -5,  -4, -30,   4, 0 }, /* ligne 7 */
//		{ 0,  50,   4,  16,  12,  12,  16,   4,  50, 0 }, /* ligne 8 */
//		{ 0,   0,   0,   0,   0,   0,   0,   0,   0, 0 }, /* ligne 9 */
//	};

	/* score d'une case jouee par un joueur blanc */
	int scoreAdverse [10][10] = {
		{ 0,     0,   0,   0,   0,   0,   0,   0,     0, 0 }, /* ligne 0 */
		{ 0, -2000,  90, -20, -15, -15, -20,  90, -2000, 0 }, /* ligne 1 */
		{ 0,    80, 200,  10,  30,  30,  10, 100,    90, 0 }, /* ligne 2 */
		{ 0, -  20,  10,   0,   0,   0,   0,  10, -  20, 0 }, /* ligne 3 */
		{ 0, -  15,  30,   0,   0,   0,   0,  30, -  15, 0 }, /* ligne 4 */
		{ 0, -  15,  30,   0,   0,   0,   0,  30, -  15, 0 }, /* ligne 5 */
		{ 0, -  20,  10,   0,   0,   0,   0,  10, -  20, 0 }, /* ligne 6 */
		{ 0,    80, 200,  10,  30,  30,  10, 100,    90, 0 }, /* ligne 7 */
		{ 0, -2000,  90, -20, -15, -15, -20,  90, -2000, 0 }, /* ligne 8 */
		{ 0,     0,   0,   0,   0,   0,   0,   0,     0, 0 }, /* ligne 9 */
	};

//	int scoreAdverse [10][10] = {
//		{ 0,    0,   0,   0,   0,   0,   0,   0,    0, 0 }, /* ligne 0 */
//		{ 0, -900, 220, -90, -70, -70, -90, 220, -900, 0 }, /* ligne 1 */
//		{ 0,  220, 200,  20,   0,   0,  20, 200,  220, 0 }, /* ligne 2 */
//		{ 0,  -90,  30, -30, -40, -20, -40,  30,  -90, 0 }, /* ligne 3 */
//		{ 0,  -78,   0, -40,   0,   0, -20,   0,  -78, 0 }, /* ligne 4 */
//		{ 0,  -78,   0, -20,   0,   0, -40,   0,  -78, 0 }, /* ligne 5 */
//		{ 0,  -90,  30, -40, -20, -40, -30,  30,  -90, 0 }, /* ligne 6 */
//		{ 0,  220, 200,  20,   0,   0,  20, 200,  220, 0 }, /* ligne 7 */
//		{ 0, -900, 220, -90, -70, -70, -90, 220, -900, 0 }, /* ligne 8 */
//		{ 0,    0,   0,   0,   0,   0,   0,   0,    0, 0 }, /* ligne 9 */
//	};

//	int scoreAdverse [10][10] = {
//		{ 0,   0,   0,   0,   0,   0,   0,   0,   0, 0 }, /* ligne 0 */
//		{ 0, -50,  -4, -16, -12, -12, -16,  -4, -50, 0 }, /* ligne 1 */
//		{ 0,  -4,  30,   4,   5,   5,   4,  30,  -4, 0 }, /* ligne 2 */
//		{ 0, -16,   4,  -1,   0,   0,  -1,   4, -16, 0 }, /* ligne 3 */
//		{ 0, -12,   5,   0,   0,   0,   0,   5, -12, 0 }, /* ligne 4 */
//		{ 0, -12,   5,   0,   0,   0,   0,   5, -12, 0 }, /* ligne 4 */
//		{ 0, -16,   4,  -1,   0,   0,  -1,   4, -16, 0 }, /* ligne 3 */
//		{ 0,  -4,  30,   4,   5,   5,   4,  30,  -4, 0 }, /* ligne 2 */
//		{ 0, -50,  -4, -16, -12, -12, -16,  -4, -50, 0 }, /* ligne 1 */
//		{ 0,   0,   0,   0,   0,   0,   0,   0,   0, 0 }, /* ligne 9 */
//	};

	int note = 0;      /* note attribuee au damier */
	int caseX, caseY ; /* cases du damier */

	typeStructureCoups listeNoir[NOMBRE_COUPS]; /* coups possibles pour les noirs */
	typeStructureCoups listeBlanc[NOMBRE_COUPS]; /* coups possibles pour les blancs */

	int coupsNoir= 0;    /* Nombre de coups pour les Noirs */
	int coupsBlanc = 0;  /* Nombre de coups pour les Blancs */
	int majoritaire = 0; /* Determination du gagnant */


	/* on determine si le jeu est fini */
	/* dans ce cas, si les noirs gagnent, note = MAX_NOTE sinon note = -MAX_NOTE */

	coupsNoir  = coup_possibles(damier, listeNoir, NOIR);
	coupsBlanc = coup_possibles(damier, listeBlanc, BLANC);

	if (coupsNoir == 0 && coupsBlanc == 0)
	{
		/* on determine qui est le gagnant */
		for (caseX = 1; caseX < 9; caseX++)
		{
			for (caseY = 1; caseY < 9; caseY++)
			{
				if ( damier[caseX][caseY] == NOIR )
					majoritaire++;
				else
					majoritaire--;
			}
		}

		if (majoritaire < 0)
			return (-MAX_NOTE); /* BLANC GAGNANT */
		else if (majoritaire > 0)
			return (MAX_NOTE);  /* NOIR GAGNANT */
		else
			return (0);         /* EGALITE */
	}

	/* Calcul de la note du damier */
	for (caseX = 1; caseX < 9; caseX++)
	{
		for (caseY = 1; caseY < 9; caseY++)
		{
			if ( damier[caseX][caseY] == NOIR )
				note += scoreJoueur[caseX][caseY];

			if ( damier[caseX][caseY] == BLANC )
				note += scoreAdverse[caseX][caseY];
		}
	}

	/* on affine la note en fonction de la mobilite des joueurs */
	note += (COEFF_MOBILITE*(coupsNoir - coupsBlanc));

	return note;
}


/*************************************************************************/
/* Nom : ordinateur_debutant                                             */
/*                                                                       */
/* Role : Fonction permettant de determiner le coup de l'ordinateur      */
/*        pour un niveau debutant                                        */
/*                                                                       */
/* Entree : couleurOrdinateur : couleur de l'ordinateur                  */
/*                                                                       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 23/02/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 02/03/1997 | Suppression du parametre couleurJoueur             */
/*     |            | prise en compte de la suppression du parametre     */
/*     |            |                                                    */
/* PDB | 20/04/1997 | Modification de la facon de jouer un coup          */
/*     |            |                                                    */
/*************************************************************************/


void ordinateur_debutant ( int couleurOrdinateur )
{

	char caseTrouve = FALSE;   /* l'ordinateur peut jouer une case */
	char caseX, caseY;         /* case du damier */
	char coupX, coupY;	   /* coup joue par l'ordinateur */


	/* on attribut des lettres a chaque case du 	*/
	/* damier                                       */
	/*						*/
	/*   a b c d e f g h 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 1| | |B|B|B|B| | | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 2| | |C|C|C|C| | | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 3|B|C|A|A|A|A|C|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 4|B|C|A| | |A|C|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 5|B|C|A| | |A|C|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 6|B|C|A|A|A|A|C|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 7| | |C|C|C|C| | | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 8| | |B|B|B|B| | | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/*						*/
	/* L'ordinateur ordonnera ses mouvements selon  */
	/* le type de la case                           */
	/* l'ordre des mouvements sera :                */
	/* - les coins                                  */
	/* - puis les cases A-B-C       		*/
	/* - et en dernier lieu la premiere case libre  */

	/* On verifie les coins */

	if (retourne(G_board, 1, 1, TRUE, couleurOrdinateur))
	{
		coupX = 1;
		coupY = 1;
		caseTrouve = TRUE;
	}

	if (!caseTrouve && retourne(G_board, 8, 8, TRUE, couleurOrdinateur))
	{
		coupX = 8;
		coupY = 8;
		caseTrouve = TRUE;
	}

	if (!caseTrouve && retourne(G_board, 1, 8, TRUE, couleurOrdinateur))
	{
		coupX = 1;
		coupY = 8;
		caseTrouve = TRUE;
	}

	if (!caseTrouve && retourne(G_board, 8, 1, TRUE, couleurOrdinateur))
	{
		coupX = 8;
		coupY = 1;
		caseTrouve = TRUE;
	}

	/* on verifie les cases au centres -> case type A */

	caseX = 3;

	while ((!caseTrouve) && (caseX < 7))
	{
		caseY = 3;
		while ((!caseTrouve) && (caseY < 7))
		{
			if (retourne(G_board, caseX, caseY, TRUE, couleurOrdinateur))
			{
				coupX = caseX;
				coupY = caseY;
				caseTrouve = TRUE;
			}
			caseY++;
		}
		caseX++;

	}

	/* On verifie les cases exterieures -> case type B */

	caseY = 3;

	while ((!caseTrouve) && (caseY < 7))
	{
		if (retourne(G_board, 1, caseY, TRUE, couleurOrdinateur))
		{
			coupX = 1;
			coupY = caseY;
			caseTrouve = TRUE;
		}

		if (!caseTrouve && retourne(G_board, 8, caseY, TRUE, couleurOrdinateur))
		{
			coupX = 8;
			coupY = caseY;
			caseTrouve = TRUE;
		}

		if (!caseTrouve && retourne(G_board, caseY, 1, TRUE, couleurOrdinateur))
		{
			coupX = caseY;
			coupY = 1;
			caseTrouve = TRUE;
		}

		if (!caseTrouve && retourne(G_board, caseY, 8, TRUE, couleurOrdinateur))
		{
			coupX = caseY;
			coupY = 8;
			caseTrouve = TRUE;
		}

		caseY++;
	}

	/* On verifie les cases interieures -> type C */

	caseY = 3;

	while ((!caseTrouve) && (caseY < 7))
	{
		if (retourne(G_board, 2, caseY, TRUE, couleurOrdinateur))
		{
			coupX = 2;
			coupY = caseY;
			caseTrouve = TRUE;
		}

		if (!caseTrouve && retourne(G_board, 7, caseY, TRUE, couleurOrdinateur))
		{
			coupX = 7;
			coupY = caseY;
			caseTrouve = TRUE;
		}

		if (!caseTrouve && retourne(G_board, caseY, 2, TRUE, couleurOrdinateur))
		{
			coupX = caseY;
			coupY = 2;
			caseTrouve = TRUE;
		}

		if (!caseTrouve && retourne(G_board, caseY, 7, TRUE, couleurOrdinateur))
		{
			coupX = caseY;
			coupY = 7;
			caseTrouve = TRUE;
		}

		caseY++;
	}

	/* On prend la premire case disponible */

	caseX = 1;

	while ((!caseTrouve) && (caseX < 9))
	{
		caseY = 1;
		while ((!caseTrouve) && (caseY < 9))
		{
			if (retourne(G_board, caseX, caseY, TRUE, couleurOrdinateur))
			{
				coupX = caseX;
				coupY = caseY;
				caseTrouve = TRUE;
			}
			caseY++;
		}
		caseX++;

	}

	/* arrive ici si l'ordinateur ne peut joueur une case, il doit passer sont tour */

	if (caseTrouve)
	{

		/* on affiche le coup joue */

		jouer_coup(coupX, coupY, couleurOrdinateur);

		/* on inclus le coup dans les statistiques */

		G_statistique[couleurOrdinateur].coups.x = coupX;
		G_statistique[couleurOrdinateur].coups.y = coupY;
	}
	else
		dialogue_passe_tour(couleurOrdinateur);

}


/*************************************************************************/
/* Nom : ordinateur_intermediaire                                        */
/*                                                                       */
/* Role : Fonction permettant de determiner le coup de l'ordinateur      */
/*        pour un niveau intermediaire                                   */
/*                                                                       */
/* Entree : couleurOrdinateur : couleur de l'ordinateur                  */
/*                                                                       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 24/02/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 02/03/1997 | Suppression du parametre couleurJoueur             */
/*     |            | prise en compte de la suppression du parametre     */
/*     |            |                                                    */
/* PDB | 18/04/1997 | Ajout de statistique (coup joue)                   */
/*     |            |                                                    */
/*************************************************************************/


void ordinateur_intermediaire ( int couleurOrdinateur )
{

	char caseTrouve = FALSE  ; /* l'ordinateur peut jouer une case */
	char caseX, caseY        ; /* case du damier */
	int  coups               ; /* variable de boucle */
	char damierTmp[10][10]   ; /* damier tamporaire */
	char score[NOMBRE_COUPS] ; /* score en fonction du coup joue */
	int  i,j                 ; /* compteur pour les boucles */
	int  borne, maxScore     ;

	/* on attribut des lettres a chaque case du 	*/
	/* damier                                       */
	/*						*/
	/*   a b c d e f g h 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 1| |C|A|B|B|A|C| | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 2|C|X|F|G|G|F|X|C| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 3|A|F|D|E|E|D|F|A| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 4|B|G|E| | |E|G|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 5|B|G|E| | |E|G|B| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 6|A|F|D|E|E|D|F|A| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 7|C|X|F|G|G|F|X|C| 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/* 8| |C|A|B|B|A|C| | 				*/
	/*  +-+-+-+-+-+-+-+-+ 				*/
	/*						*/
	/* L'ordinateur ordonnera ses mouvements selon  */
	/* le type de la case                           */
	/* l'ordre des mouvements sera :                */
	/* - les coins                                  */
	/* - puis les cases A-D-B-E-G-F-C-X		*/


	typeStructureCoups ordreCoups[NOMBRE_COUPS]=
		{ {1,1}, {1,8}, {8,1}, {8,8},						 		/* les coins       */
		  {1,3}, {1,6}, {3,1}, {6,1}, {8,3}, {8,6}, {3,8}, {6,8}, 	/* cases de type A */
		  {3,3}, {3,6}, {6,3}, {6,6}, 								/* cases de type D */
		  {1,4}, {1,5}, {4,1}, {5,1}, {8,4}, {8,5}, {4,8}, {5,8}, 	/* cases de type B */
		  {3,4}, {3,5}, {4,3}, {5,3}, {4,6}, {5,6}, {6,4}, {6,5}, 	/* cases de type E */
		  {2,4}, {2,5}, {4,2}, {5,2}, {7,4}, {7,5}, {4,7}, {5,7}, 	/* cases de type G */
		  {2,3}, {2,6}, {3,2}, {6,2}, {3,7}, {6,7}, {7,3}, {7,6}, 	/* cases de type F */
		  {1,2}, {1,7}, {2,1}, {7,1}, {2,8}, {7,8}, {8,2}, {8,7}, 	/* cases de type C */
		  {2,2}, {2,7}, {7,2}, {7,7}             	    }; 			/* cases de type X */


	/* Nombre de cases par type                      */
	/*                                               */
	/*               coins  A  D  B  E  G  F  C  X   */
	/*                   |  |  |  |  |  |  |  |  |   */

	char typeCase[9] = { 4, 8, 4, 8, 8, 8, 8, 8, 4 };


	/* on met a zero les scores */

	memset(score, 0, NOMBRE_COUPS);


	/* dans l'ordre des mouvements, on determine la case que l'ordinateur va jouer */

	for (coups = 0; coups < NOMBRE_COUPS; coups++)
	{
		/* copie du damier courant */

		memcpy(damierTmp, G_board, 100);

		if (retourne(damierTmp, ordreCoups[coups].x, ordreCoups[coups].y, TRUE, couleurOrdinateur))
		{
			damierTmp[ordreCoups[coups].x][ordreCoups[coups].y] = couleurOrdinateur;

			/* on determine le score du coup */

			for(caseX = 1; caseX < 9; caseX++)
			{
				for(caseY = 1; caseY < 9; caseY++)
				{
					if (damierTmp[caseX][caseY]==couleurOrdinateur)
						score[coups]++;
				}
			}

		}
	}

	/* on determine le meilleur coup dans le premier type de case possible */

	borne = 0;

	for ( i = 0; ((i < 9) && (!caseTrouve)); i++)
	{
		maxScore = 0;

		for (j = borne; j < (borne + typeCase[i]); j++)
		{
			if (score[j] > maxScore)
			{
				caseX = ordreCoups[j].x;
				caseY = ordreCoups[j].y,

				maxScore = score[j];
				caseTrouve = TRUE;
			}
		}
		borne += typeCase[i] ; /* on passe au prochain type */
	}

	/* on valide la case que l'ordinateur joue */

	if (caseTrouve)
	{
		/* le retournement des pions est possible */

		retourne(G_board, caseX, caseY, TRUE, couleurOrdinateur);

		/* on affiche le coup joue */

		jouer_coup(caseX, caseY, couleurOrdinateur);

		/* on inclus le coup dans les statistiques */

		G_statistique[couleurOrdinateur].coups.x = caseX;
		G_statistique[couleurOrdinateur].coups.y = caseY;


//		G_board[caseX][caseY] = couleurOrdinateur;
	}
	else
		dialogue_passe_tour(couleurOrdinateur);


}

/*************************************************************************/
/* Nom : ordinateur_expert                                               */
/*                                                                       */
/* Role : Fonction permettant de determiner le coup de l'ordinateur      */
/*        pour un niveau expert                                          */
/*                                                                       */
/* Entree : couleurOrdinateur : couleur de l'ordinateur                  */
/*          couleurJoueur     : couleur du joueur                        */
/*          profondeur        : profondeur de la recherche               */
/*                              (<=> au niveau de l'expert)              */
/*                                                                       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 02/03/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 18/04/1997 | Ajout de statistique (coup joue)                   */
/*     |            |                                                    */
/* PDB | 21/05/1997 | Restriction de la zone du curseur de la souris lors*/
/*     |            | de la recherche en profondeur                      */
/*     |            |                                                    */
/* PDB | 22/06/1997 | Ajout possibilite d'interrompre la recursivite     */
/*     |            |                                                    */
/* PDB | 29/06/1997 | Suppression interruption de la recursivite         */
/*     |            |                                                    */
/* PDB | 30/06/1997 | Ajout message lorsque reflexion d'Otho             */
/*     |            |                                                    */
/*************************************************************************/

void ordinateur_expert ( int couleurOrdinateur, int profondeur )
{
	int scoreCoup = 0;
	char caseX, caseY;


	/* affichage du curseur durant la reflexion de l'ordinateur */
	change_curseur(ORDINATEUR);

	mouse_visible(TRUE);

	/* On restreint la zone de la souris */
	mousestatus(&G_sourisX, &G_sourisY, &G_sourisBouton);
	mouserangeset(0, 0, 639, 440);
	mouselocset(G_sourisX, G_sourisY);


	/* vidage du buffer clavier */
	while(kbhit())
		getch();


	/* recherche du meilleur coup avec la methode minmax ou alpha_beta */
	G_autoriseArret = FALSE;

	G_stopAlphaBeta = FALSE;
//	dialogue_aide_en_ligne("Touche <ECHAP> pour interrompre la recherche.");
	dialogue_aide_en_ligne("Je rflchis, patientez svp...");

//	min_max (G_board, profondeur, couleurOrdinateur, &caseX, &caseY, &scoreCoup);

	alpha_beta (G_board, profondeur, couleurOrdinateur, &caseX, &caseY, &scoreCoup, 0, 0);

	dialogue_aide_en_ligne("");

	/* On supprime la restriction de la zone de la souris */
	mousestatus(&G_sourisX, &G_sourisY, &G_sourisBouton);
	mouserangeset(0, 0, 639, 479);
	mouselocset(G_sourisX, G_sourisY);

	/* masquage du curseur de la souris */
	change_curseur(couleurOrdinateur);
	mouse_visible(FALSE);

	retourne(G_board, caseX, caseY, TRUE, couleurOrdinateur);

	/* on affiche le coup joue */

	jouer_coup(caseX, caseY, couleurOrdinateur);

	/* on inclus le coup dans les statistiques */

	G_statistique[couleurOrdinateur].coups.x = caseX;
	G_statistique[couleurOrdinateur].coups.y = caseY;

}

/*************************************************************************/
/* Nom : min_max                                                         */
/*                                                                       */
/* Role : Fonction permettant de determiner le meilleur coup en          */
/*        cherchant en profondeur en garantissant d'avoir dans le Pieme  */
/*        coups a venir (P=profondeur) une note egale a l'un des coups   */
/*        joues a la profondeur P                                        */
/*                                                                       */
/* Entree : board          : damier courant                              */
/*          profondeur     : profondeur de la recherche                  */
/*          joueur         : joueur jouant le coup courant               */
/*          coupsX, coupsY : coup joue par le joueur                     */
/*          score          : score du coup                               */
/*                                                                       */
/*          les entrees coupsX, coupsY et score sont utilises pour les   */
/*          appels recursifs  la fonction min_max                        */
/*                                                                       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 02/03/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/*     |            |                                                    */
/*************************************************************************/


void min_max ( char board[10][10],
	       int  profondeur   ,
	       int  joueur       ,
	       char *coupsX      ,
	       char *coupsY      ,
	       int  *score        )
{
	int i; /* variable de boucle */
	int le_score, nombreCoups; /* variables de recherche */
	char buffer[MAX_BUFFER];

	char new_board[10][10]; /* nouveau damier pour memoriser les etats */

	typeStructureCoups liste[NOMBRE_COUPS], le_coup; /* coups possibles et coup a jouer */



	G_appel++;

	drwstring(1, TEXTE, FOND, "Nombre de situations analyses :            ", 140, 454);
	sprintf(buffer, "Nombre de situations analyses : %ld", G_appel);
	drwstring(1, TEXTE, FOND, buffer, 140, 454);

	/* on determine si un des deux joueurs peut jouer */

	if ((nombreCoups = coup_possibles(board,liste,joueur)) == 0)
		if ((nombreCoups = coup_possibles(board, liste, joueur = -joueur+3)) == 0)
		{
			/* le joueur est bloque */
			*score = evaluation_damier(board);
			return;  /* On va donc inverser le joueur (cas Max-Max ou Min-Min) */
		}


	if (joueur == NOIR)
		*score = -MAX_NOTE;
	else
		*score = MAX_NOTE;

	/* une profondeur de moins pour la recherche */

	profondeur--;

	/* on etudie tous les coups que le joueur puisse jouer */

	for (i = 0; i < nombreCoups; i++)
	{
		/* copie du damier courant vers le nouveau damier */
		memcpy(new_board, board, 100);

		/* on joue le coup */

		retourne(new_board, liste[i].x, liste[i].y, TRUE, joueur);
		new_board[liste[i].x][liste[i].y] = joueur;

		if (profondeur)

			/* ici on est a un noeud de l'arbre */

			min_max(new_board, profondeur, -joueur+3, &le_coup.x , &le_coup.y, &le_score);
		else
			/* ici, on est aux feuilles de l'arbre. */

			le_score = evaluation_damier(new_board);


		if (joueur == NOIR) /* Procedure Max en marche */
		{
			if (le_score >= *score)
			{
				*score = le_score;
				*coupsX = liste[i].x;
				*coupsY = liste[i].y;
			}
		}
		else /* Procedure Min en marche */
			if (le_score <= *score)
			{
				*score = le_score;
				*coupsX = liste[i].x;
				*coupsY = liste[i].y;
			}
	}
}

/*************************************************************************/
/* Nom : alpha_beta                                                         */
/*                                                                       */
/* Role : Fonction permettant de determiner le meilleur coup en          */
/*        cherchant en profondeur en garantissant d'avoir dans le Pieme  */
/*        coups a venir (P=profondeur) une note egale a l'un des coups   */
/*        joues a la profondeur P                                        */
/*                                                                       */
/* Entree : board          : damier courant                              */
/*          profondeur     : profondeur de la recherche                  */
/*          joueur         : joueur jouant le coup courant               */
/*          coupsX, coupsY : coup joue par le joueur                     */
/*          score          : score du coup                               */
/*          note           : note du coup                                */
/*          qui            : utile pour la generation de l'arbre         */
/*                                                                       */
/*          les entrees coupsX, coupsY et score sont utilises pour les   */
/*          appels recursifs de la fonction alpha_beta                   */
/*                                                                       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 18/04/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 22/06/1997 | Possibilite d'interrompre la recursivite en        */
/*     |            | appuyant sur la touche ESC                         */
/*     |            |                                                    */
/*************************************************************************/


void alpha_beta ( char board[10][10],
		  int  profondeur   ,
		  int  joueur       ,
		  char *coupsX      ,
		  char *coupsY      ,
		  int  *score       ,
		  int  note         ,
		  int  qui )
{
	int i; /* variable de boucle */
	int le_score, nombreCoups; /* variables de recherche */
//	char buffer[MAX_BUFFER];

	char new_board[10][10]; /* nouveau damier pour memoriser les etats */

	typeStructureCoups liste[NOMBRE_COUPS], le_coup; /* coups possibles et coup a jouer */

	int toucheClavier; /* touche appuyee par l'utilisateur */



	G_appel++;

	/* gestion du temps */
	gere_reflexion();
//	if ((G_appel % 43) == 0 )
//	{
//		drwstring(1, TEXTE, FOND, "Nombre de situations analyses :            ", 140, 454);
//		sprintf(buffer, "Nombre de situations analyses : %ld", G_appel);
//		drwstring(1, TEXTE, FOND, buffer, 140, 454);
//	}

	/* on determine si un des deux joueurs peut jouer */

	if ((nombreCoups = coup_possibles(board,liste,joueur)) == 0)
		if ((nombreCoups = coup_possibles(board, liste, joueur = -joueur+3)) == 0)
		{
			/* le joueur est bloque */
			*score = evaluation_damier(board);
			return;  /* On va donc inverser le joueur (cas Max-Max ou Min-Min) */
		}


	if (joueur == NOIR)
		*score = -MAX_NOTE;
	else
		*score = MAX_NOTE;

	/* une profondeur de moins pour la recherche */

	profondeur--;

	/* on etudie tous les coups que le joueur puisse jouer */

	for (i = 0; i < nombreCoups; i++)
	{
		/* copie du damier courant vers le nouveau damier */
		memcpy(new_board, board, 100);

		/* on joue le coup */

		retourne(new_board, liste[i].x, liste[i].y, TRUE, joueur);
		new_board[liste[i].x][liste[i].y] = joueur;

		/* Arret de la recursivite provoque par l'utilisateur */
		/* => appui sur la touche ESC */

		if (iskey()) /* appui clavier */
		{
			toucheClavier = getkey();

			if ( toucheClavier == ESC && G_autoriseArret )
				G_stopAlphaBeta = TRUE;
		}

		if (profondeur && !G_stopAlphaBeta)

			/* ici on est a un noeud de l'arbre */

			alpha_beta(new_board, profondeur, -joueur+3, &le_coup.x , &le_coup.y, &le_score, *score, joueur);
		else
			/* ici, on est aux feuilles de l'arbre. */

			le_score = evaluation_damier(new_board);


		if (joueur == NOIR) /* Procedure Max en marche */
		{
			if (qui == BLANC && le_score >= note)
			{
				/* coupe Min -> Max */
				i = nombreCoups;
				*score = MAX_NOTE;
			}
			else if (le_score >= *score)
			{
				*score = le_score;
				*coupsX = liste[i].x;
				*coupsY = liste[i].y;

				if ( *score == MAX_NOTE )
					i = nombreCoups; /* fin de la coupe */
			}
		}
		else /* Procedure Min en marche */
		{
			if (qui == NOIR && le_score <= note)
			{
				/* coupe Max -> Min */
				i = nombreCoups;
				*score = -MAX_NOTE;
			}

			else if (le_score <= *score)
			{
				*score = le_score;
				*coupsX = liste[i].x;
				*coupsY = liste[i].y;

				if ( *score == -MAX_NOTE )
					i = nombreCoups; /* fin de la coupe */
			}
		}
	}
}

