/*************************************************************************/
/* Nom : allocation_memoire                                              */
/*                                                                       */
/* Role : Fonction permettant d'allouer la memoire pour l'application    */
/*                                                                       */
/* Entree : -                                                            */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 15/12/1996 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 13/01/1997 | Ajout de la gestion des erreurs                    */
/*     |            |                                                    */
/* PDB | 06/04/1997 | Modification de la taille d'allocation de          */
/*     |            | G_ptrFond                                          */
/*     |            |                                                    */
/*     |            |                                                    */
/*************************************************************************/

#include <alloc.h>
#include <stdio.h>

#include "include\svgacc.h"
#include "include\prototyp.h"

#include "include\global.h"

void allocation_memoire ( void )
{
	int i;

	printf("[allocation_memoire] Test mmoire...");

	/* allocation de la memoire pour les chiffres graphiques de 1  0 */
	/* chaque chiffre est defini dans une matrice de 20*28 pixels     */
	/* chaque chiffre occupe donc 20*28 = 560 octets                  */
	/* a cela il faut rajouter 4 octets pour la definition de la      */
	/* taille : 564 <=> TAILLE_CHIFFRE                                */

	for (i=0; i < NOMBRE_CHIFFRE; i++)
	{
		G_ptrNombre[i] = (RasterBlock far *) malloc(TAILLE_CHIFFRE);

		if (G_ptrNombre[i] == NULL)
			erreur_application("allocation_memoire", "Erreur allocation memoire", 2);

	}

	/* allocation memoire pour memoriser un chiffre graphique tampon */

	G_ptrNombreTmp = (RasterBlock far *)malloc(TAILLE_CHIFFRE);

	if (G_ptrNombreTmp == NULL)
		erreur_application("allocation_memoire", "Erreur allocation memoire", 3);



	/* allocation de la memoire pour les pions du jeu */

	/* chaque pion est defini dans une matrice de 42*42 pixels */
	/* chaque indicateur occupe donc 42*42 = 1764 octets         */
	/* a cela il faut rajouter 4 octets pour la definition de la */
	/* taille : 1768 <=> TAILLE_PION                             */

	for (i = 0; i < NB_TYPE_CASE; i++)
	{
		G_ptrPionJeu[i] = (RasterBlock far *)malloc(TAILLE_PION);

		if (G_ptrPionJeu[i] == NULL)
			erreur_application("allocation_memoire", "Erreur allocation memoire", 4);
	}

	/* allocation memoire pour un pion tamporaire */

	G_ptrPion = (RasterBlock far *)malloc(TAILLE_PION);

	if (G_ptrPion == NULL)
		erreur_application("allocation_memoire", "Erreur allocation memoire", 5);

	/* allocation memoire pour memoriser une zone graphique du jeu */

	G_ptrFond = (RasterBlock far *)malloc(20000);

	if (G_ptrFond==NULL)
		erreur_application("allocation_memoire", "Erreur allocation memoire", 6);

	/* allocation de la memoire pour les indicateurs de choix         */
	/* chaque indicateur est defini dans une matrice de 17*16 pixels  */
	/* chaque indicateur occupe donc 17*16 = 272 octets               */
	/* a cela il faut rajouter 4 octets pour la definition de la      */
	/* taille : 276 <=> TAILLE_INDICATEUR                             */

	for (i = 0; i < NOMBRE_CHOIX+1; i++)
	{
		G_ptrMenu[i] = (RasterBlock far *) malloc (TAILLE_MENU);
	}

	/* Arrive ici tout est OK */

	printf("OK\n");
}

/*************************************************************************/
/* Nom : libere_memoire                                                  */
/*                                                                       */
/* Role : Fonction permettant de liberer la memoire allouee              */
/*                                                                       */
/* Entree : -                                                            */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 20/12/1996 | Creation de la fonction.                           */
/*     |            |                                                    */
/*************************************************************************/


void libere_memoire ( void )
{
	int i; /* variables de boucle */

	/* nombres graphiques */

	for (i=0; i < NOMBRE_CHIFFRE; i++)
		free(G_ptrNombre[i]);


	/* type de cases */

	for (i=0; i < NB_TYPE_CASE; i++)
		free(G_ptrPionJeu[i]);

	/* liberation de la memoire des choix du menu */

	for (i = 0; i < NOMBRE_CHOIX+1; i++)
	{
        free(G_ptrMenu[i]);
	}
	
	/* memoire tampon ds objets graphiques */

	free(G_ptrPion);
	free(G_ptrNombreTmp);
	free(G_ptrFond);
}