/* menupal.c - manage the main menu */

/* Copyright 1997-2001 Otho team. */

/*
modification history
--------------------
15a,21oct01,pdb  based on Otho version 1.4b.
*/

/*
DESCRIPTION
This module manages the main menu of the game.
The choice can be one of the following options:
Initiation - Learn Otho game
Jouer      - play a game
Visualiser - Show a old game
A propos   - Information about this game
Regles     - Rules of the game
Ouvertures - start game database
Scores     - High-scores depending of the game level
Quitter    - Exit the game
*/

/* includes */
   
#include <conio.h>
#include <stdio.h>

#include "include\svgacc.h"

#include "include\prototyp.h"
#include "include\global.h"
#include "include\keyboard.h"


#include "include\menupal.h"

/*******************************************************************************
*
* menu_principal - manage the main menu
*
* This routine allows to manage the main menu of the Otho game.
*
* RETURNS: N/A.
*
*/
	   
void menu_principal( void )
	{
	char buffer[MAX_BUFFER]   ;
	int  i                    ;	/* variables utilisees pour les boucles */
	char finJeu        =  0   ;	/* variable indiquant la fin du jeu */
	char toucheReturn  =  0   ;	/* appui sur la touche entree ? */
	int  toucheClavier =  0   ;	/* valeur de la touche du clavier */
	char choixCourant  = -1   ;	/* choix courant du menu general */
	char choixAncien   = -1   ;	/* choix precedent du menu general */
	char horsZone             ;	/* est-on en dehors des zones de menus */
	char oldZone       = FALSE;	/* une difference par rapport au dernier emplacement du curseur */

	/* on charge provisoirement l'image du menu general */
	/* afin de memoriser les libelles d'une autre couleur */

	sprintf(buffer, "%s", "images\\menu_2.pcx\0");
	charge_image(buffer, RES640);

	video_off();

	/* memorisation en memoire libelles en coloration plus claires */
	/* les differents libelles sont situes en coordonnees zoneLibelleMenu */

	for (i = 0; i < NOMBRE_CHOIX; i++)
	{
		blkget(zoneLibelleMenu[i].x1,
		       zoneLibelleMenu[i].y1,
		       zoneLibelleMenu[i].x2,
		       zoneLibelleMenu[i].y2,
		       G_ptrMenu[i]);

	}

	/* Gestion des choix du menu general */

	while (!finJeu)
		{
		mouse_visible(FALSE); /* non affichage de la souris */
		mousecursorset(&curseurSourisMenu);

		/* on charge l'image definitive du menu general */
		sprintf(buffer, "%s", "images\\menu_1.pcx\0");
		charge_image(buffer, RES640);

		/* affichage progressif du menu principal */
//		palioauto(G_gifPalette, 0, 255, -128);
//		video_on();
//		palioauto(G_gifPalette, 0, 255, 30);


		mouse_visible(TRUE); /* affichage de la souris */

		/* vidage du buffer clavier */
		while (kbhit())	getch();

		while (!toucheReturn)
			{
			/*******************************/
			/* Partie gestion de la souris */
			/*******************************/

			horsZone = TRUE;

			mousestatus(&G_sourisX, &G_sourisY, &G_sourisBouton);

			/* On realise une capture si clic sur le bouton droit */
			if (G_sourisBouton == 2)
				capture_ecran();

			for ( i = 0; i < NOMBRE_CHOIX; i++)
				{
				/* On regarde si la souris est situee sur l'un des choix possibles */
				if (mouse_zone(zoneMenu[i].x1, zoneMenu[i].x2, zoneMenu[i].y1, zoneMenu[i].y2))
					{
					horsZone = FALSE;
					oldZone = horsZone;

					if (G_sourisBouton & BOUTON_GAUCHE)	 /* click gauche souris */
						{
						toucheReturn = 1;
						}
					if (choixCourant != i) /* affichage des indicateurs si nouvelle position */
						{
						choixCourant = i;

						drwstring(1, 16, 0, commentMenu[NOMBRE_CHOIX], 60, 430);
						drwstring(1, 16, 0, commentMenu[i], 60, 430);

						affiche_choix(choixAncien, choixCourant);
						choixAncien = choixCourant;
						}
					}
				}


			/* on verifie si on sort d'un des choix du menu general */
			if ( horsZone == TRUE && oldZone != horsZone )
				{
				oldZone = horsZone;

				/* effacement de la ligne de commentaires */
				drwstring(1, 16, 0, commentMenu[NOMBRE_CHOIX], 60, 430);

				/* on efface le choix courant */
				choixCourant = -1;
				affiche_choix(choixAncien, choixCourant);
				choixAncien = -1;
				}

			/*****************************/
			/* Partie gestion du clavier */
			/*****************************/

			if (iskey()) /* appui clavier */
				toucheClavier = getkey();

			if (toucheClavier > 0)
				{
				switch (toucheClavier)
					{
					case RETURN:

						toucheReturn = 1;
						break;

					case FLECHE_HAUT:

						choixCourant--;
						if (choixCourant == -1)
							choixCourant = NOMBRE_CHOIX - 1;

						affiche_choix(choixAncien, choixCourant);

						choixAncien = choixCourant;
						break;

					case FLECHE_BAS :

						choixCourant++;
						if (choixCourant == NOMBRE_CHOIX)
							choixCourant = 0;
						affiche_choix(choixAncien, choixCourant);
						choixAncien = choixCourant;
						break;
					}
				toucheClavier = 0;
				}
			}
		toucheReturn = 0;

		/* effacement du menu general */
		mouse_visible(FALSE);
		palioauto(G_gifPalette, 0, 255, -30);


		/* on redirige les traitements en fonction du choix */
		switch (choixCourant)
			{
			case 0:	/* choix initiation */
				jouer(INITIATION);
				break;

			case 1:	/* choix jouer */
				jouer(NORMAL);
				break;

			case 2:	/* choix visualiser */
				visualiser();
				break;

			case 3:	/* choix a propos... */
				apropos();
				break;

			case 4:	/* choix regles */
				regles();
				break;

			case 5:	/* choix ouverture */
				ouverture();
				break;

			case 6:	/* choix scores */
				affiche_high_score(ECRAN_MENU_GENERAL);
				break;

			case 7:	/* choix quitter */
				finJeu = 1;
				break;
			
			default:	/* non valid choice -> do nothing */
				break;

			}

		choixCourant  = -1;	/* choix courant du menu general   */
		choixAncien   = -1;	/* choix precedent du menu general */

		}

	}

/*******************************************************************************
*
* affiche_choix - display the current choice
*
* This routine displays the current choice.
*
* RETURNS: N/A.
*
*/
	   
void affiche_choix
	(
	char ancien,	/* old choice */
	char courant	/* new choice */
	)
	{
	mouse_visible(FALSE);
	
	/* effacement des indicateurs de l'ancien choix */
	if (ancien != -1)
		spriteput(SET, 9, zoneLibelleMenu[ancien].x1, zoneLibelleMenu[ancien].y1, G_ptrMenu[NOMBRE_CHOIX]);

	/* affichage choix courant en memorisant au prealable le libelle precedent */
	if (courant != -1)
		spritegap(9, zoneLibelleMenu[courant].x1, zoneLibelleMenu[courant].y1, G_ptrMenu[courant], G_ptrMenu[NOMBRE_CHOIX]);
	
	mouse_visible(TRUE);
	}
