/*************************************************************************/
/* Nom : ordinateur_vs_ordinateur                                        */
/*                                                                       */
/* Role : Fonction permettant a deux ordinateurs de jouer contre eux     */
/*                                                                       */
/*                                                                       */
/* Entree : -                                                            */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 24/02/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 20/04/1997 | Amelioration de l'environnement de jeu             */
/*     |            |                                                    */
/* PDB | 06/05/1997 | Mise en place de la reprise d'une ancienne partie  */
/*     |            |                                                    */
/* PDB | 21/05/1997 | Deport de le gestion d'une reprise de partie       */
/*     |            |                                                    */
/*************************************************************************/

#include <dos.h>
#include <conio.h>
#include <stdio.h>
#include <string.h>

#include "include\svgacc.h"

#include "include\prototyp.h"
#include "include\global.h"
#include "include\couleur.h"

void ordinateur_vs_ordinateur ( char reprise )
{
	char finJeu     = 0;      /* drapeau indiquant la fin du jeu */
	int  nombrePion = 0;      /* nombre de pions places sur le damier */
	int  profondeur;          /* profondeur de la recherche du meilleur coup */
	char clicStop    = FALSE; /* clique sur le bouton STOP */
	char clicSuivant = FALSE; /* clique sur le bouton > */
	char afficheCoup = FALSE; /* affichage des coups possibles */

	if (reprise == NOK) /* partie inexistante */
	{
	    sprintf(G_nom[NOIR] , "OTHO NOIR" );
	    sprintf(G_nom[BLANC], "OTHO BLANC");
	}

	/* affichage des Noms des joueurs */
	drwstring(1, TEXTE_NOM, FOND, G_nom[NOIR],   20+(90-strlen(G_nom[NOIR])*8)/2, 75);
	drwstring(1, TEXTE_NOM, FOND, G_nom[BLANC], 530+(90-strlen(G_nom[BLANC])*8)/2, 75);


	if (reprise == NOK)
	{
		/* debut du jeu */
		dialogue_aide_en_ligne("C'est au joueur NOIR de commencer.");
		statistique(NOIR);
	}
	else /* affichage du dernier coup joue */
	{
		G_jeuIndiceCrt = G_jeuIndiceMax - 1;
		coup_suivant();

		/* raz aide en ligne */
		dialogue_aide_en_ligne("Reprise d'un partie.");
	}


	/* Tant que l'on ne quitte pas le jeu */
	do
	{
		/* tableau de commande a l'etat normal */
		initialise_tableau_commande();


		if (clicSuivant)
		{
			clicSuivant = FALSE;

			switch(G_choixTypeJeu[G_joueurCourant-1])
			{
				case 1: /* niveau dbutant */
					G_statistique[G_ordinateur].profondeur = 0;
					ordinateur_intermediaire(G_joueurCourant);
					break;

				case 2: /* niveau intermedaire */
					if (nombrePion > LIMITE_INTERMEDIAIRE)
						profondeur = 64 - nombrePion;
					else
						profondeur = PROFONDEUR_INTERMEDIAIRE;

					G_statistique[G_joueurCourant].profondeur = profondeur;
					ordinateur_expert(G_joueurCourant, profondeur);
					break;

				case 3: /* niveau confirme */
					if (nombrePion > LIMITE_CONFIRME)
						profondeur = 64 - nombrePion;
					else
						profondeur = PROFONDEUR_CONFIRME;

					G_statistique[G_joueurCourant].profondeur = profondeur;
					ordinateur_expert(G_joueurCourant, profondeur);
					break;

				case 4: /* niveau expert */
					if (nombrePion > LIMITE_EXPERT)
						profondeur = 64 - nombrePion;
					else
						profondeur = PROFONDEUR_EXPERT;

					G_statistique[G_joueurCourant].profondeur = profondeur;
					ordinateur_expert(G_joueurCourant, profondeur);
					break;

				default:
					break;
			}

			/* on recalcule les statistiques */
			statistique(G_joueurCourant);

			/* on affiche la nouvelle disposition des pions */
			affiche_pions();

			/* on memorise le jeu courant */
			memorise_jeu(G_joueurCourant);

			/* on passe au joueur suivant */
			change_joueur();

		} /* fin si clicSuivant */

		/********************************/
		/* attente appuie bouton souris */
		/********************************/
		mouse_attente();

		/* Raz zone d'aide en ligne */

		dialogue_aide_en_ligne("");

		/* Gestion effet appuye    */

		/* bouton coup precedent */
		if (mouse_zone(162, 173, 18, 38))
		{
			dialogue_bouton_appuye(162, 18, 11, 20);
			drwstring(1, TEXTE, FOND, "<", 164, 22);
		}

		/* bouton coup suivant */
		if (mouse_zone(178, 189, 18, 38))
		{
			dialogue_bouton_appuye(178, 18, 11, 20);
			drwstring(1, TEXTE, FOND, ">", 180, 22);
		}

		/* bouton conseil */
		if (mouse_zone(198, 260, 18, 38))
		{
			dialogue_bouton_appuye(198, 18, 62, 20);
			drwstring(1, TEXTE, FOND, "Conseil", 201, 22);
		}

		/* bouton stop */
		if (mouse_zone(440, 476, 18, 38))
		{
			dialogue_bouton_appuye(440, 18, 36, 20);
			drwstring(1, TEXTE, FOND, "Stop", 443, 22);
		}

		/* On efface les coups possibles */
		if (afficheCoup)
		{
			afficheCoup = FALSE;
			affiche_coup(G_joueurCourant, FALSE);
		}

		/*********************************/
		/* attente relache bouton souris */
		/*********************************/
		mouse_attente_relache();

		/* bouton coup precedent */
		if (mouse_zone(162, 173, 18, 38))
			coup_precedent();

		/* bouton coup suivant */
		if (mouse_zone(178, 189, 18, 38))
		{
			if (G_jeuIndiceCrt == G_jeuIndiceMax)
				clicSuivant = TRUE;
			else
			    coup_suivant();
		}

		/* bouton conseil */
		if (mouse_zone(198, 260, 18, 38))
		{
			affiche_coup(G_joueurCourant, TRUE);
			afficheCoup = TRUE;
		}

		/* on desire arreter la partie */
		if (mouse_zone(440, 476, 18, 38))
			clicStop = TRUE;

		/* on desire passer son tour */
		if(mouse_zone(300, 350,  10,  60))
			change_joueur();


		/* la partie est-elle terminee ? */
		nombrePion = G_statistique[NOIR].score + G_statistique[BLANC].score;

		if(nombrePion == 64 || !G_statistique[NOIR].score || !G_statistique[BLANC].score || clicStop)
		{
			fin_jeu();
			finJeu = 1;
		}

	} while(!finJeu); /* Fin du jeu a deux */

	/* sauvegarde de la partie jouee si au moins un coup a ete joue */
	if (G_jeuIndiceCrt > 0)
		sauve_partie();
}
