/* scores.c - manage the high-scores screen */

/* Copyright 1997-2001 Otho team. */

/*
modification history
--------------------
15b,25oct01,pdb  fixed bug about minutes display.
15a,21oct01,pdb  based on Otho version 1.4b.
*/

/*
DESCRIPTION
This module manages the high-scores display
It is called when the choice is scores from
the main menu
*/

/* includes */

#include <dos.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "include\global.h"
#include "include\prototyp.h"
#include "include\couleur.h"


/*******************************************************************************
*
* high_score - computes the high-scores table
*
* This routine is called from the GAME screen and it updates the high-scores
* table depending on the results.
*
* RETURNS: N/A.
*
*/

void high_score ( void )
	{
	char deltaScore ; /* difference entre le score noir et le score blanc */
	int  gagnant    ; /* joueur ayant gagne la partie */
	char niveau     ; /* niveau de la partie */
	char delta      ; /* pour la recherche d'un rang dans les highs scores */
	char rang = -1  ; /* rang dans la liste */
	char i          ; /* variable de boucle */
	struct date dateJeu; /* date du jeu */


	/* Ecart entre les scores */
	deltaScore = G_statistique[NOIR].score - G_statistique[BLANC].score;

	/* Pas de high score possible si match nul */
	if (deltaScore == 0)
		return;

	/* On determine le gagnant */
	if (deltaScore > 0)
		gagnant = NOIR;

	if (deltaScore < 0)
		gagnant = BLANC;


	deltaScore = abs(deltaScore); /* transformation en nombre positif */

	/* on determine le niveau du jeu */
	if (G_ordinateur == VIDE)
		niveau = 5;	/* C'etait un jeu entre humain */
	else
		{
		niveau = G_choixTypeJeu[G_ordinateur-1]; /* jeu humain contre ordinateur */

		/* Pas de classement pour l'ordinateur pour preserver l'honneur */
		/* du joueur humain */
		if (gagnant == G_ordinateur)
			return;
		}

	/* Recherche d'une place dans le classement */
	for (i=0; i < 5; i++)
		{
		delta = abs(G_highScore[niveau][i].scoreNoir - G_highScore[niveau][i].scoreBlanc);
		if (deltaScore > delta)
			{
			rang = i; /* on memorise le rang */
			break;	  /* sorti de la boucle for */
			}
		}

	/* le gagnant est classe */
	if (rang != -1)
		{
		/* decalage dans les autres lignes */
		i = 4; /* on part de la fin du classement */
		while (i > rang)
			{
			strcpy(G_highScore[niveau][i].nomJoueur,G_highScore[niveau][i-1].nomJoueur);
			G_highScore[niveau][i].scoreNoir  = G_highScore[niveau][i-1].scoreNoir;
			G_highScore[niveau][i].scoreBlanc = G_highScore[niveau][i-1].scoreBlanc;
			G_highScore[niveau][i].jourJeu    = G_highScore[niveau][i-1].jourJeu;
			G_highScore[niveau][i].moisJeu    = G_highScore[niveau][i-1].moisJeu;
			G_highScore[niveau][i].anneeJeu   = G_highScore[niveau][i-1].anneeJeu;
			G_highScore[niveau][i].temps      = G_highScore[niveau][i-1].temps;

			i--; /* on grimpe d'une ligne */
			}

		/* on affecte le rang du gagnant */
		getdate(&dateJeu); /* on recupere la date du systeme */

		strcpy(G_highScore[niveau][rang].nomJoueur, G_nom[gagnant]);
		G_highScore[niveau][rang].scoreNoir  = G_statistique[NOIR].score;
		G_highScore[niveau][rang].scoreBlanc = G_statistique[BLANC].score;
		G_highScore[niveau][rang].jourJeu    = dateJeu.da_day;
		G_highScore[niveau][rang].moisJeu    = dateJeu.da_mon;
		G_highScore[niveau][rang].anneeJeu   = dateJeu.da_year;
		G_highScore[niveau][rang].temps      = G_temps[gagnant];

		/* on memorise le rang et le niveau */
		G_memoriseRang = rang;
		G_memoriseNiveau = niveau;

		/* sauvegarde des scores */
		sauve_score();

		/* affichage des high_scores */
		affiche_high_score(ECRAN_JEU);

		}
	}

/*******************************************************************************
*
* affiche_high_score - displays the high-scores
*
* This routine displays the high-scores on the screen. Parameter <flagEcran>
* determines from which screen the high-scores is invoked (GAME or MAIN MENU)
*
* RETURNS: N/A.
*
*/

void affiche_high_score
	(
	char flagEcran	/* screen source flag */
	)
	{
	char    buffer[MAX_BUFFER];	/* chaine tampon */
	char    image[MAX_BUFFER]; /* image a charger */
	int     x, y;	/* coordonnees */
	int     taille;
	int     niveau;
	int     rang;
	time_t  temps;
	int     minutes;
	int     hours;
	int     seconds;

	char texteNiveau[6][MAX_BUFFER] = {

		"Niveau initiation",
		"Niveau debutant",
		"Niveau intermediaire",
		"Niveau confirme",
		"Niveau expert",
		"Niveau 'Humain contre Humain'"};


	/* fichier image du jeu */
	sprintf(image, "%s", "images\\otho.pcx\0");

	if ( flagEcran == ECRAN_MENU_GENERAL )
		/* on vient du menu general */
		/* on charge l'image contenant la palette de couleur qui nous interesse */
		charge_image(image, RES640);

	/**************************/
	/* Preparation de l'ecran */
	/**************************/

	video_off();

//	palioauto(G_gifPalette, 0, 255, -128); /* image invisible */

//	fillscreen(0); /* ecran noir */

	if ( flagEcran == ECRAN_MENU_GENERAL )
		video_on();	/* autorisation de l'affichage */

	/* dessin du contour de l'ecran */
	dialogue_boite(5, 5, 635, 475);


	/* Affichage du texte concernant le meilleurs scores */
	sprintf(buffer, "Les meilleurs joueurs d'%s !", G_version);

	taille = strlen(buffer) * 8;
	x = (640 - taille) / 2;
	y = 20;

	drwfillbox(1, FOND, x-5, y, x+taille, y+20);
	dialogue_cadre(x-5, y, x+taille, y+20);
	drwstring(1, TEXTE_NOM, FOND, buffer, x, y+4);

	/* Ligne separant l'ecran en deux */
	drwline(1, BORD2, 305+10, 60, 305+10, 440);
	drwline(1, BORD1, 306+10, 60, 306+10, 440);


	/* Affichage des classements par niveau et par rang */
	y = 40;	/* premiere ligne pour les classements */

	for (niveau = 0; niveau < 6; niveau++)
		{
		/* colonne droite ou gauche */
		if (niveau >= 3)
			{
			x = 320+10;
			} else
			x = 20;

		/* si colonne droite, on revient a la premiere ligne */
		if (niveau == 3)
			y = 40;

		/* Affichage du niveau */
		y += 20;
		drwstring(1, TEXTE_NOM, FOND, texteNiveau[niveau], x, y);
		drwline(1, BORD2, x, y+14, x+strlen(texteNiveau[niveau])*8, y+14);
		drwline(1, BORD1, x, y+15, x+strlen(texteNiveau[niveau])*8, y+15);
		y += 5;

		/* classement par niveau */
		for (rang = 0; rang < 5; rang++)
			{
			/* colonne droite ou gauche */
			if (niveau >= 3)
				{
				x = 340+10-20;
				} else
				x = 40-20;

			/* ligne suivante */
			y += 20;

			/* Rang du niveau correspondant*/
			sprintf(buffer, "%i", rang+1);
			drwstring(1, TEXTE_NOM, FOND, buffer, x, y);


			/* Nom du joueur */
			x += 10;
			statistique_affiche_donnee(x, y, G_highScore[niveau][rang].nomJoueur, 10);

			if ( flagEcran == ECRAN_JEU && G_memoriseNiveau == niveau && G_memoriseRang == rang )
				drwstring(1, TEXTE_SCORE, TEXTE_FOND, G_highScore[niveau][rang].nomJoueur, x+1, y);

			/* Score Noir */
			x += 90;
			sprintf(buffer, "%i", G_highScore[niveau][rang].scoreNoir);
			statistique_affiche_donnee(x, y, buffer, 2);

			if ( flagEcran == ECRAN_JEU && G_memoriseNiveau == niveau && G_memoriseRang == rang )
				drwstring(1, TEXTE_SCORE, TEXTE_FOND, buffer, x+1, y);

			/* tiret */
			x += 20;
			sprintf(buffer, "-");
			drwstring(1, TEXTE_NOM, FOND, buffer, x, y);

			/* Score Blanc */
			x += 10;
			sprintf(buffer, "%i", G_highScore[niveau][rang].scoreBlanc);
			statistique_affiche_donnee(x, y, buffer, 2);

			if ( flagEcran == ECRAN_JEU && G_memoriseNiveau == niveau && G_memoriseRang == rang )
				drwstring(1, TEXTE_SCORE, TEXTE_FOND, buffer, x+1, y);

			/* Date du jeu */
			x += 30;
			sprintf(buffer, "%2i/%2i/%4i", G_highScore[niveau][rang].jourJeu, G_highScore[niveau][rang].moisJeu, G_highScore[niveau][rang].anneeJeu);
			statistique_affiche_donnee(x, y, buffer, 10);

			/* Temps de jeu */
			x += 90;

			/* conversion du temps en chaine de caractere */

			temps   = G_highScore[niveau][rang].temps;
			hours   = (int) (temps / 3600);
			temps  -= (hours*3600);
			minutes = (int) (temps / 60);
			temps  -= (minutes*60);
			seconds = (int) temps;

			if (hours > 0)	/* game duration was above 1 hour ? */
				sprintf(buffer, "99:99", minutes, seconds);
			else
				sprintf(buffer, "%02i:%02i", minutes, seconds);

			statistique_affiche_donnee(x, y, buffer, 5);

			if ( flagEcran == ECRAN_JEU && G_memoriseNiveau == niveau && G_memoriseRang == rang )
				drwstring(1, TEXTE_SCORE, TEXTE_FOND, buffer, x+1, y);


			}
		y += 5;	/* espacement avant affichage du niveau */
		}

	/* affichage progressif de l'ecran */
//	palioauto(G_gifPalette, 0, 255, 100);

	/* Bouton OK et gestion de ce bouton */
//	dialogue_OK(590, 450, 622, 465);
	dialogue_OK(300, 450, 332, 465);
//	dialogue_OK(560, 20+16, 592, 35+16);

	if ( flagEcran == ECRAN_JEU && G_memoriseNiveau != 0)
		{
		/* Sauvegarde de la partie */
		sauve_partie();

		/* on ecrase l'image par elle meme pour effacer les high scores */
//		palioauto(G_gifPalette, 0, 255, -50);
		charge_image(image, RES640);

		initialise_jeu();

//		palioauto(G_gifPalette, 0, 255, -128);
		video_on();
		palioauto(G_gifPalette, 0, 255, 50);

		}

	}

/*******************************************************************************
*
* sauve_score - save the high-scores
*
* This routine saves the current high-scores to a file named scores.dat
*
* RETURNS: N/A.
*
*/

void sauve_score ( void )
	{
	FILE *fichierJeu;		 /* fichier de sauvegarde */
	char buffer[MAX_BUFFER]; /* chaine tampon */
	char nomFichier[] = "data\\scores.dat";	 /* nom du fichier complet */


	if (G_protectionEcriture == TRUE )
		{
		return;
		}

	/* on cree le fichier de sauvegarde */
	if ((fichierJeu = fopen(nomFichier, "wb")) == NULL)
		{
		sprintf(buffer, "Impossible de crer %s", nomFichier);
		dialogue(buffer);
		return;
		}

	/* sauvegarde des high scores */
	fwrite(G_highScore, sizeof(G_highScore), 1, fichierJeu);

	fclose(fichierJeu);
	}


/*******************************************************************************
*
* charge_score - load the high-scores
*
* This routine loads the current high-scores from a file named scores.dat
*
* RETURNS: OK if loads is successful, else NOK if file does not exist
* or a problem occurs while reading the file.
*
*/

char charge_score ( void )
	{
	FILE *fichierJeu;
	char nomFichier[] = "data\\scores.dat";	 /* nom du fichier complet */

	/* on ouvre le fichier des high scores */
	if ((fichierJeu = fopen(nomFichier, "rb")) == NULL)
		return NOK;

	/* on se positionne en debut de fichier */
	fseek(fichierJeu, 0, SEEK_SET);

	/* chargement des scores */
	fread(G_highScore, sizeof(G_highScore), 1, fichierJeu);

	fclose(fichierJeu);

	return OK;
	}