#include <mem.h>
#include <stdio.h>

#include "include\svgacc.h"

#include "include\prototyp.h"
#include "include\global.h"
#include "include\couleur.h"

/*************************************************************************/
/* Nom : statistique                                                     */
/*                                                                       */
/* Role : Fonction permettant de calculer les statistiques des differents*/
/*        joueur et de realiser l'affichage vers les zones adequats      */
/*                                                                       */
/*                                                                       */
/* Entree : joueurCourant : couleur du joueur demandant de recalculer    */
/*                          les statistiques                             */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 16/04/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/*     |            |                                                    */
/*     |            |                                                    */
/*************************************************************************/

void statistique ( int joueurCourant )
{
	int evaluation;    /* resultat de l'evaluation du damier   */

	int nombreCoups;   /* nombre de coups possibles */
	int joueurAdverse; /* le joueur adverse */

	typeStructureCoups liste[NOMBRE_COUPS]; /* les coups possibles */


	joueurAdverse = -joueurCourant + 3;

	/************************/
	/* Scores des joueurs   */
	/************************/

	/* on compte le nombre de pion de chaque joueur et on affiche le score */

	compte_pions();

	/************************/
	/* Mobilite des joueurs */
	/************************/

	/* on determine le nombre de coups possible de chaque joueur */

	/* Joueur Courant */
	nombreCoups = coup_possibles(G_board, liste, joueurCourant);

	G_statistique[joueurCourant].mobilite = nombreCoups;

	/* Joueur Adverse */
	nombreCoups = coup_possibles(G_board, liste, joueurAdverse);

	G_statistique[joueurAdverse].mobilite = nombreCoups;

	/****************************************/
	/* Evaluation du coup du joueur courant */
	/****************************************/

	evaluation = evaluation_damier (G_board);

	G_statistique[joueurCourant].evaluation = evaluation;

	/****************************************************************/
	/* Nombre de feuilles analysees (recherche min-max, alpha_beta) */
	/****************************************************************/

	G_statistique[joueurCourant].nombreFeuilles = G_appel;

	G_appel = 0;

	/* Affichage des statistiques pour les NOIRS */
	statistique_affiche(NOIR, 20, 170+15);

	/* Affichage des statistiques pour les BLANCS */
	statistique_affiche(BLANC, 530, 170+15);

}

/*************************************************************************/
/* Nom : statistique_affiche                                             */
/*                                                                       */
/* Role : Fonction permettant de realiser l'affichage des statistiques   */
/*                                                                       */
/*                                                                       */
/* Entree : joueurCourant : couleur du joueur demandant de recalculer    */
/*                          les statistiques                             */
/*	    x, y          : coordonnees de la premiere statistique       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 27/04/1997 | Creation de la fonction.                           */
/*     |            |                                                    */
/* PDB | 18/05/1997 | Factorisation du code pour l'affichage des         */
/*     |            | differentes donnees de statistique                 */
/*     |            |                                                    */
/*************************************************************************/

void statistique_affiche ( int joueur, int x, int y )
{
	char buffer[20]; /* chaine tampon */
	char lettres [9][2] = {" ", "A", "B", "C", "D", "E", "F", "G", "H"};
	char chiffres[9][2] = {" ", "1", "2", "3", "4", "5", "6", "7", "8"};


	/****************************************************************/
	/* Affichage des statistiques pour le joueur passe en parametre */
	/****************************************************************/

	/* Coup joue */
	drwstring(1, TEXTE, FOND, "Coup :", x, y);

	/* affichage coup joue */
	sprintf(buffer, "%s%s", lettres[G_statistique[joueur].coups.x], chiffres[G_statistique[joueur].coups.y]);
	statistique_affiche_donnee(x+60, y, buffer, 2);


	/* Mobilite */
	drwstring(1, TEXTE, FOND, "Mobilit:", x, y+25);

	/* affichage mobilite */
	sprintf(buffer, "%d", G_statistique[joueur].mobilite);
	statistique_affiche_donnee(x, y+45, buffer, 2);


	/* Evaluation */
	drwstring(1, TEXTE, FOND, "Evaluation:", x, y+70);

	/* affichage evaluation */
	sprintf(buffer, "%d", G_statistique[joueur].evaluation);
	statistique_affiche_donnee(x, y+90, buffer, 5);


	/* Donnees concernant les resultats de la recherche min_max ou alpha_beta */
	/* Pas de traitement si pas de recherche min_max ou alpha_beta */

	if (G_statistique[joueur].nombreFeuilles != 0)
	{

		/* Nombre de feuilles */
		drwstring(1, TEXTE, FOND, "Feuilles:", x, y+115);

		/* affichage nombre de feuilles */
		sprintf(buffer, "%ld", G_statistique[joueur].nombreFeuilles);
		statistique_affiche_donnee(x, y+135, buffer, 11);


		/* profondeur */
		drwstring(1, TEXTE, FOND, "Profondeur:", x, y+160);

		/* affichage profondeur */
		sprintf(buffer, "%d", G_statistique[joueur].profondeur);
		statistique_affiche_donnee(x, y+180, buffer, 2);

	}

}


/*************************************************************************/
/* Nom : statistique_affiche_donnee                                      */
/*                                                                       */
/* Role : Fonction permettant d'afficher une donnee de statistique       */
/*        (cette fonction s'inspire de dialogue_zone_saisie)             */
/*                                                                       */
/* Entree : x1, y1 : coordonnees superieures gauche du cadre             */
/*          texte  : texte a afficher                                    */
/*          taille : taille en caractere de la zone de donnees           */
/*                                                                       */
/* Sortie : -                                                            */
/*                                                                       */
/* ===================================================================== */
/* Nom | Date       | Description                                        */
/* ----+------------+--------------------------------------------------- */
/* PDB | 18/05/1997 | Creation de la fonction a partir de la fonction    */
/*     |            | dialogue_zone_saisie                               */
/*     |            |                                                    */
/*************************************************************************/

void statistique_affiche_donnee ( int x1, int y1, char *texte, int taille)
{

	int x2, y2;

	x2 = x1 + (taille)*8;
	y2 = y1 + 13;

	/* dessin du fond     */
	/*                    */
	/* x1,y1              */
	/*	+---+         */
	/*      |   |         */
	/*      +---+         */
	/*           x2,y2    */

	drwfillbox(1, TEXTE_FOND, x1, y1, x2, y2);


	/* dessin du contour  */

	drwbox(1, BORD2, x1-1, y1-1, x2+1, y2+1);


	/* affichage du texte */

	drwstring(1, TEXTE, TEXTE_FOND, texte, x1+1, y1);


}
