/* video.c - manage screen video */

/* Copyright 1997-2001 Otho team. */

/*
modification history
--------------------
15a,21oct01,pdb  based on Otho version 1.4b.
*/

/*
DESCRIPTION
This module manges the current display
*/

/* includes */
   
#include <dos.h>
#include <stdio.h>
#include "include\prototyp.h"
#include "include\global.h"

/* locals */

static int L_resolution = -1; /* resolution courante */

/* globals */


/*******************************************************************************
*
* depart_video - restore video context mode
*
* RETURNS: N/A.
*
*/
	   
void depart_video (void)
	{
	restext(); /* mode texte 80x25 */
	videomodeset(G_strCarteVideo.modeVideoDepart); /* contexte de depart */
	}


/*******************************************************************************
*
* video_on - set video to visible
*
* RETURNS: N/A.
*
*/
	   
void video_on (void)
	{
	videoon();
	}

/*******************************************************************************
*
* video_off - set video to invisible
*
* RETURNS: N/A.
*
*/

void video_off (void)
	{
	videooff();
	}

/*******************************************************************************
*
* loadpal - convert a data palette
*
* This routine converts an unknown 8-bit palette to 6-bit palette
*
* RETURNS: N/A.
*
*/

void loadpal
	(
	PaletteData pal,	/* palette to convert */
	int nbCol			/* number of colors */
	)
	{
	int  a, fixit = 0;
	char red, green, blue; /* couleurs */


	/* On determine si l'on doit faire une conversion 8 bits -> 6 bits */

	for (a = 0 ; a < nbCol && !fixit ; a++)
		{
		red   = pal[a].r ;
		green = pal[a].g ;
		blue  = pal[a].b ;

		/* 6 bits => 111111 binaire = 3F hexa (63 decimal) */

		if (red > 0x3F || green > 0x3F || blue > 0x3F)
			fixit=1;
		}

	/* On realise un decalage de 2 bits -> palette sur 6 bits */

	if (fixit)
		{
		for ( a = 0 ; a < nbCol;a++)
			{
			pal[a].r >>= 2;
			pal[a].g >>= 2;
			pal[a].b >>= 2;
			}
		}
	}

/*******************************************************************************
*
* charge_image - load an image
*
* This routine load an pcx image
*
* RETURNS: N/A.
*
*/

void charge_image
	(
	char	*nomImage,	/* file name */
	int		resolution	/* screen resolution */
	)
	{
	int     ok = 0;
	int     xSize, ySize, numCol; /* caracteristique de l'image taille en x et y, nombre de couleurs utilises */
	char    buffer[MAX_BUFFER];	  /* buffer de construction de chaine */

	/* A-t-on une evolution dans la resolution ? */

	if (L_resolution != resolution )
		{
		L_resolution = resolution;

		/* mode video en fonction de la resolution choisie */

		switch (resolution)
			{
			case RES1024:
				ok = res1024();	/* resolution 1024x768 */
				break;
			case RES800 :
				ok = res800();	/* resolution  800x600 */
				break;
			case RES640 :
				ok = res640();	/* resolution  640x480 */
				break;
			case RES320 :
				ok = res320();	/* resolution  320x200 */
				break;
			}



		if (!ok)
			erreur_application("charge_image", "Mode video impossible", 300);

		}

	/* ecran noir si ce n'est pas le cas */

	video_off();

	/* initialisation souris */

	mouseenter();
	mouse_visible(FALSE); /* masquage de la souris */


	/* informations de l'image a charger */

	ok = pcxgetinfo(nomImage, &xSize, &ySize, &numCol, G_gifPalette);


	if (!ok)
		{
		sprintf(buffer, "Impossible de charger %s", nomImage);

		if (G_autoriseArretApplication == TRUE)
			erreur_application("charge_image", buffer, 301);
		else
			{
			/* autorisation arret pour la prochaine fois */
			G_autoriseArretApplication = TRUE;
			return;
			}
		}

	loadpal(G_gifPalette, numCol); /* conversion palette 8 bits -> 6 bits */


	palset(G_gifPalette, 0, 255);
	palget(G_gifPalette, 0, 255);


	/* chargement de l'image */

	ok = pcxput(SET, 0, 0, nomImage);

	if (!ok)
		{
		sprintf(buffer, "Impossible de charger %s", nomImage);
		erreur_application("charge_image", buffer, 302);
		}

	}
